#!/usr/bin/python3.5
#
# This is the Python module implementing access to the bmp180 temperature and
# barometric pressure sensor
# 2-line LCD display
# U. Raich UCC, Nov.2017


# read the calibration parameter from the register
#
#	Parameter  |  MSB    |  LSB    |  bit
# ------------|---------|---------|-----------
#		AC1    |  0xAA   | 0xAB    | 0 to 7
#		AC2    |  0xAC   | 0xAD    | 0 to 7
#		AC3    |  0xAE   | 0xAF    | 0 to 7
#		AC4    |  0xB0   | 0xB1    | 0 to 7
#		AC5    |  0xB2   | 0xB3    | 0 to 7
#		AC6    |  0xB4   | 0xB5    | 0 to 7
#		B1     |  0xB6   | 0xB7    | 0 to 7
#		B2     |  0xB8   | 0xB9    | 0 to 7
#		MB     |  0xBA   | 0xBB    | 0 to 7
#		MC     |  0xBC   | 0xBD    | 0 to 7
#		MD     |  0xBE   | 0xBF    | 0 to 7
#
import pigpio as pigpio
import sys,time

I2C_BUS                =    1
I2C_FLAGS              =    0
BMP180_ADDRESS         = 0x77
handle                 =   -1
pi                     =   -1

#
# bmp180 power modes:
#
ULTRA_LOW_POWER       = 0
STANDARD_POWER        = 1
HIGH_RESOLUTION       = 2
ULTRA_HIGH_RESOLUTION = 3

BMP180_AC1_REG         =0xaa
BMP180_AC2_REG         =0xac
BMP180_AC3_REG         =0xae
BMP180_AC4_REG         =0xb0
BMP180_AC5_REG         =0xb2
BMP180_AC6_REG         =0xb4

BMP180_B1_REG          =0xb6
BMP180_B2_REG          =0xb8

BMP180_MB_REG          =0xba
BMP180_MC_REG          =0xbc
BMP180_MD_REG          =0xbe

BMP180_OUT_XLSB_REG    =0xf8
BMP180_OUT_LSB_REG     =0xf7
BMP180_OUT_MSB_REG     =0xf6

BMP180_CTRL_MEAS_REG   =0xf4

BMP180_SOFT_RESET      =0xe0
BMP180_CHIP_ID_REG     =0xd0

BMP180_DO_MEAS         =0x2e


class Bmp180Class:
    AC1 = -1
    AC2 = -1
    AC3 = -1
    AC4 = -1
    AC5 = -1
    AC6 = -1
    B1  = -1
    B2  = -1
    MB  = -1
    MC  = -1
    MD  = -1
    oversampling            = -1
    __samplingWait          = -1
    __samplingWaitTable     = []

    uncompensatedTemperature = -1
    uncompensatedPressure    = -1
    
    temperature             = -1
    pressure                = -1
    debug                   = False
    
    def __init__(self):
#        
# open the connection to the pigpiod daemon
#
        global pi
        global handle
        
        pi = pigpio.pi()
        if not pi.connected:
            print("Could not connect to the pigpiod daemon, giving up...")
            exit()
#
# open the I2C bus driver
#
        handle = pi.i2c_open(I2C_BUS,
                             BMP180_ADDRESS,I2C_FLAGS)
        if handle < 0:
            print("Could not open the I2C bus for address ",
                  BMP180_ADDRESS," giving up...")
            exit()
        else:
            self.handle = handle
#
# read calibration data
#
        retCode = pi.i2c_read_byte_data(handle,BMP180_AC1_REG)
        if retCode < 0:
            print("in init: cannot read calibraion data. Giving up...")
            exit()
        self.AC1 = retCode << 8
        
        retCode = pi.i2c_read_byte_data(handle,BMP180_AC1_REG+1)
        if retCode < 0:
            print("in init: cannot read calibraion data. Giving up...")
            exit()       
        else:
            self.AC1 |= retCode
            if self.debug:
                print("in init: AC1 = 0x"+format(self.AC1,'04x'),"successfully read")
            self.AC1 = self.__toSignedShort(self.AC1)
            if self.debug:
                print("in init: AC1 = ",self.AC1)
#
# if I can read the first calibration register I should be able to read
# the others as well
# Therefore I skip error checking for the other regs
#
        self.AC2 = pi.i2c_read_byte_data(handle,BMP180_AC2_REG) << 8
        self.AC2 |= pi.i2c_read_byte_data(handle,BMP180_AC2_REG+1)
        if self.debug:
            print("in init: AC2 = 0x"+format(self.AC2,'04x'),"successfully read")
        self.AC2 = self.__toSignedShort(self.AC2)
        if self.debug:       
            print("in init: AC2 = ",self.AC2)
        
        self.AC3 = pi.i2c_read_byte_data(handle,BMP180_AC3_REG) << 8
        self.AC3 |= pi.i2c_read_byte_data(handle,BMP180_AC3_REG+1)
        if self.debug:
            print("in init: AC3 = 0x"+format(self.AC3,'04x'),"successfully read")
        self.AC3 = self.__toSignedShort(self.AC3)
        if self.debug:
            print("in init: AC3 = ",self.AC3)
#
# A4, A5, A6 are unsigned shorts
#
# all the other calibration values are signed shorts
#         
        self.AC4 = pi.i2c_read_byte_data(handle,BMP180_AC4_REG) << 8
        self.AC4 |= pi.i2c_read_byte_data(handle,BMP180_AC4_REG+1)
        if self.debug:
            print("in init: AC4 = 0x"+format(self.AC4,'04x'),"successfully read")
            print("in init: AC4 = ",self.AC4)

        self.AC5 = pi.i2c_read_byte_data(handle,BMP180_AC5_REG) << 8
        self.AC5 |= pi.i2c_read_byte_data(handle,BMP180_AC5_REG+1)
        if self.debug:
            print("in init: AC5 = 0x"+format(self.AC5,'04x'),"successfully read")
            print("in init: AC5 = ",self.AC5)        
       
        self.AC6 = pi.i2c_read_byte_data(handle,BMP180_AC6_REG) << 8
        self.AC6 |= pi.i2c_read_byte_data(handle,BMP180_AC6_REG+1)
        if self.debug:
            print("in init: AC6 = 0x"+format(self.AC6,'04x'),"successfully read")
            print("in init: AC6 = ",self.AC6)        
               
        self.B1 = pi.i2c_read_byte_data(handle,BMP180_B1_REG) << 8
        self.B1 |= pi.i2c_read_byte_data(handle,BMP180_B1_REG+1)
        if self.debug:
            print("in init: B1  = 0x"+format(self.B1,'04x'),"successfully read")
        self.B1 = self.__toSignedShort(self.B1)
        if self.debug:
            print("in init: B1 = ",self.B1)  
       
        self.B2 = pi.i2c_read_byte_data(handle,BMP180_B2_REG) << 8
        self.B2 |= pi.i2c_read_byte_data(handle,BMP180_B2_REG+1)
        if self.debug:
            print("in init: B2  = 0x"+format(self.B2,'04x'),"successfully read")
        self.B2 = self.__toSignedShort(self.B2)
        if self.debug:
            print("in init: B2 = ",self.B2)  
       
        self.MB = pi.i2c_read_byte_data(handle,BMP180_MB_REG) << 8
        self.MB |= pi.i2c_read_byte_data(handle,BMP180_MB_REG+1)
        if self.debug:
            printrint("in init: MB  = 0x"+format(self.MB,'04x'),"successfully read")
        self.MB = self.__toSignedShort(self.MB)
        if self.debug:
            print("in init: MB = ",self.MB)  
              
        self.MC = pi.i2c_read_byte_data(handle,BMP180_MC_REG) << 8
        self.MC |= pi.i2c_read_byte_data(handle,BMP180_MC_REG+1)
        if self.debug:
            print("in init: MC  = 0x"+format(self.MC,'04x'),"successfully read")
        self.MC = self.__toSignedShort(self.MC)
        if self.debug:
            print("in init: MC = ",self.MC)
        
        self.MD = pi.i2c_read_byte_data(handle,BMP180_MD_REG) << 8
        self.MD |= pi.i2c_read_byte_data(handle,BMP180_MD_REG+1)
        if self.debug:
            print("in init: MD  = 0x"+format(self.MD,'04x'),"successfully read")
        self.MD = self.__toSignedShort(self.MD)
        if self.debug:
            print("in init: MD = ",self.MD)
        
        self.oversampling        = ULTRA_HIGH_RESOLUTION
        self.__samplingWaitTable = [0.0045,0.0075,0.0135,0.0255]
        self.__samplingWait = self.__samplingWaitTable[self.oversampling]
#
# set the calibration values from the data sheet as well as their readout
# for uncompensated temperature and pressure to test the
# conversion algorithm
#
    def dummyCalib(self):
        self.AC1 =    408
        self.AC2 =    -72
        self.AC3 = -14383
        self.AC4 =  32741
        self.AC5 =  32757
        self.AC6 =  23153
        self.B1  =   6190
        self.B2  =      4
        self.MB  = -32768
        self.MC  =  -8711
        self.MD  =   2868
        self.uncompensatedTemperature = 27898
        self.uncompensatedPressure    = 23843
        self.oversampling = ULTRA_LOW_POWER

        
#
# convert to signed short
#
    def __toSignedShort(self,val):
       if val & 0x8000 :
           return -((~val & 0xffff) +1)
       else:
           return val
       

# set oversampling
#
    def setResolution(self,oss):

        if oss < 0 or oss > 3:
            printf("in setResolution: invalid oversampling code. Skipping...")
            return
        self.oversampling = oss
        self.__samplingWait = self.__samplingWaitTable[oss]
#
# read back oversampling code
#
    def getResolution(self):
            return self.oversampling    
#
# read the chip id
#
    def chipID(self):
        global pi
        global handle
        retCode = pi.i2c_read_byte_data(handle,BMP180_CHIP_ID_REG)
        if retCode < 0:
            print("in chipID: cannot read BMP180 id")
            exit()
        else:
            return retCode
        

#
# make a measurement
#

    def measure(self):
        global pi,handle
#
# start a temperature reading
#
        retCode = pi.i2c_write_byte_data(
            handle,BMP180_CTRL_MEAS_REG,BMP180_DO_MEAS)
        if retCode < 0:
            print("in measure: Could not start measurement")
            return
        time.sleep(0.0045)      # wait for 4.5 ms
        self.uncompensatedTemperature = pi.i2c_read_byte_data(
            handle,BMP180_OUT_MSB_REG) << 8
        self.uncompensatedTemperature |= pi.i2c_read_byte_data(
            handle,BMP180_OUT_MSB_REG+1)
        if self.debug:
            print("in measure: uncompensated temperature: ",
                  self.uncompensatedTemperature)
#
# start a pressure reading
#
        cmd = 0x34 + (self.oversampling << 6)
        retCode = pi.i2c_write_byte_data(
            handle,BMP180_CTRL_MEAS_REG,cmd)
        if retCode < 0:
            print("in measure: Could not start measurement")
            return
        time.sleep(self.__samplingWait)
        self.uncompensatedPressure = pi.i2c_read_byte_data(
            handle,BMP180_OUT_MSB_REG) << 16
        self.uncompensatedPressure += pi.i2c_read_byte_data(
            handle,BMP180_OUT_MSB_REG+1) << 8
        self.uncompensatedPressure += pi.i2c_read_byte_data(
            handle,BMP180_OUT_MSB_REG+1)
        self.uncompensatedPressure >>= 8-self.oversampling 
        
        if self.debug:
            print("in measure: uncompensated pressure: ",
                  self.uncompensatedPressure)
        self.convert()
        
#
# calculate real temperature
#
    def convert(self):
        if self.debug:
            print("in convert: ------------ calculating temperature ------------")
        ut = self.uncompensatedTemperature
        if self.debug:        
            print("in convert: uncompensated temperature: ",ut)
            print("in convert: AC6: ",self.AC6," AC5: ",self.AC5)
        x1 = (ut - self.AC6) * self.AC5 >> 15
        if self.debug:
            print("in convert: x1:",x1)
        x2 = self.MC * 2048 // (x1+self.MD)
        if self.debug:
            print("in convert: x2:",x2)        
        b5 = x1 + x2
        if self.debug:
            print("in convert: b5:",b5)         
        t = (b5 + 8) >> 4
        if self.debug:
            print("in convert: calculated temperature: ",t)
        self.temperature = t/10       # in °C
#
# calculate pressure
#
        if self.debug:
            print("in convert: ------------ calculating pressure ------------")
        up = self.uncompensatedPressure
        b6 = b5 - 4000
        if self.debug:
            print("in convert: b6 =",b6)
        x1 = (self.B2 * (b6 * b6) >> 12) >> 11
        x2 = (self.AC2 * b6) >> 11
        x3 = x1 + x2    
        b3 = (((self.AC1 * 4 + x3) << self.oversampling) + 2) >> 2
        if self.debug:
            print("in convert: x1=" ,x1, " x2=",x2," x3=",x3," b3=" ,b3)
        x1 = self.AC3 * b6 >> 13
        x2 = (self.B1 * (b6 * b6) >> 12) >> 16
        x3 = ((x1 + x2) + 2) >> 2
        if self.debug:
            print("in convert: x1=" ,x1, " x2=",x2," x3=",x3)        
        b4 = self.AC4 * (x3 + 32768) >> 15
        if self.debug:
            print("in convert: b4=",b4)
        b7 = (up - b3) * (50000 >> self.oversampling)
        if self.debug:
            print("in convert: b7=",b7)
        if b7 < 0x80000000:
            p = (b7 * 2) // b4
        else:
            p = (b7//b4)*2
        if self.debug:
            print ("in convert: p=",p)
        x1 = (p>>8)*(p>>8)
        x1 = (x1*3038) >> 16
        x2 = (-7357 * p) >> 16
        if self.debug:
            print(" in convert: x1=",x1," x2=",x2)
        p = p + ((x1 + x2 + 3791) >> 4)
        if self.debug:
            print("in convert: pressure:",p)
        self.pressure = p/100    # in hPa
            
#
# get temperature
#
    def getTemperature(self):
        return self.temperature

#
# get pressure
#
    def getPressure(self):
        return self.pressure

#
# set debugging on/off
#
    def setDebug(self,onOff):
        self.debug=onOff
